import math
import random
from gmisclib import mcmc
import re

class probdist_c(object):
	def __call__(self):
		"""Sample from the probability distribution.
		@return: the sample
		@rtype: C{float}
		"""
		raise RuntimeError, "Virtual function"

	def logdens(self, x):
		"""Return the log(density) of the probability distribution at x.
		@param x: the position where the density should be evaluated.
		@type x: C{float}
		@return: C{log(density)}
		@rtype: C{float}
		@raise mcmc.NotGoodPosition: when the density is not defined at C{x}.
		"""
		raise RuntimeError, "Virtual function"


class Expo(probdist_c):
	def __init__(self, lmbda):
		assert lmbda > 0.0
		self.lmbda = lmbda

	def __call__(self):
		return random.expovariate(self.lmbda)

	def logdens(self, x):
		if x < 0.0:
			raise mcmc.NotGoodPosition, "x=%s" % str(x)
		assert x >= 0.0
		return -self.lmbda*x + math.log(self.lmbda)


class Normal(probdist_c):
	def __init__(self, mu, sigma):
		if sigma <= 0.0:
			raise mcmc.NotGoodPosition, "sigma=%s" % str(sigma)
		self.mu = mu
		self.sigma = sigma

	def __call__(self):
		return random.normalvariate(self.mu, self.sigma)

	def logdens(self, x):
		return -(((x-self.mu)/self.sigma)**2)/2.0 - math.log(math.sqrt(2*math.pi)*self.sigma)


class LogNormal(probdist_c):
	def __init__(self, mu, sigma):
		if sigma <= 0.0:
			raise mcmc.NotGoodPosition, "sigma=%s" % str(sigma)
		self.mu = mu
		self.sigma = sigma

	def __call__(self):
		return self.mu*math.exp(random.normalvariate(0.0, self.sigma))

	def logdens(self, x):
		if (x>0.0) != (self.mu>0.0):
			raise mcmc.NotGoodPosition, "x=%s, mu=%s" % (str(x), str(self.mu))
		x = math.log(x/self.mu)
		return -((x/self.sigma)**2)/2.0 - math.log(math.sqrt(2*math.pi)*self.sigma)


def logp_prior_normalized(x, guess_prob_dist):
	"""
	@type guess_prob_dist: list((str, probdist_c))
	@type x: newstem2.indexclass.index_base
	@rtype: float
	@return: the log of the prior probability.
	"""
	lp = 0.0
	gpd = [ (re.compile(pat), sampler) for (pat, sampler) in guess_prob_dist]
	for key in x.map.keys():
		v = x.p(*key)

		vv = None
		fkey = x._fmt(key)
		for (pat, sampler) in gpd:
			if pat.match(fkey):
				vv = sampler
				break
		if vv is None:
			raise KeyError, "No match found for '%s'" % fkey
		logp = vv.logdens(v)
		# print 'prior(%s;%g)=%g' % (fkey, v, logp)
		lp += logp
	return lp




from gmisclib import mcmc_helper as mcmcP
# from newstem2 import newlogger as LG

class logger_c(mcmcP.logger_template):
	"""This class is called from the Markov-Chain stepper
	and writes the log file.
	"""
	def __init__(self, logwriter, iterstep=100, ergstep=0.1):
		assert iterstep >= 0
		assert ergstep >= 0.0
		# self.lg = LG.logger(fd)
		self.lg = logwriter
		self.iterstep = iterstep
		self.ergstep = ergstep
		# self.__g_implements__ = ['logger_template']
		self.nlogged = 0
		self.erg = 0.0
		self.last_add_iter = 0
		self.last_logged_erg = None
		self.last_logged_iter = None
		self.best = None
		self.reason = 'sampled'


	def set_logstep(self, iterstep, ergstep, reason='sampled'):
		self.reason = reason
		assert iterstep > 0
		self.iterstep = iterstep
		if ergstep is None:
			self.ergstep = 0.0
		else:
			assert ergstep >= 0.0
			self.ergstep = ergstep
		self.lg.flush()


	def add(self, stepperInstance, iter):
		"""Called every step: this decides whether
		data should be logged.
		"""
		self.erg += stepperInstance.ergodic() * (iter-self.last_add_iter)
		self.last_add_iter = iter
		reason = []
		if(self.last_logged_iter is None or self.last_logged_erg is None):
			reason.append('initial')
		elif(iter>self.last_logged_iter+self.iterstep
			and self.erg>self.last_logged_erg+self.ergstep
			):
			reason.append(self.reason)
		elif self.best is None or stepperInstance.current().logp_nocompute()>self.best:
			self.best = stepperInstance.current().logp_nocompute()
			reason.append('best')
		if reason:
			self.Add(stepperInstance, iter, reason=','.join(reason))


	def Add(self, stepperInstance, iter, reason=None):
		"""Actually write a set of parameters into the log file."""
		print 'Logging', stepperInstance.current().logp_nocompute(), 'iter=', iter
		self.nlogged += 1
		self.last_logged_erg = self.erg
		self.last_logged_iter = iter
		self.lg.add('UID', stepperInstance.current().prms(),
				stepperInstance.current().pd.idxr.map,
				stepperInstance.current().logp(),
				iter,
				extra={'T': stepperInstance.acceptable.T(),
					'resetid': stepperInstance.reset_id(),
					},
				reason=reason
				)
		self.lg.flush()


	def close(self):
		self.lg.close()

	def comment(self, comment):
		self.lg.comment(comment)
		self.lg.flush()

	def header(self, k, v):
		self.lg.header(k, v)

	def headers(self, hdict):
		self.lg.headers(hdict)

	def reset(self):
		self.nlogged = 0
		self.erg = 0.0
		self.last_logged_erg = None
		self.comment("Reset")


	def set_trigger_point(self):
		self.nlogged = 0
		self.erg = 0.0
		self.last_logged_erg = None
		self.comment("run_to_bottom finished")




"""Vectors of numbers, but indexed as a dictionary.
"""

from gmisclib import gpkmisc
from gmisclib import dictops
import operator
import __builtin__


class dict_vector(dict):
	SCALARS = (int, float)
	ZERO = 0

	def copy(self):
		return dict_vector(self)


	def incr(self, k, v):
		if k in self:
			self[k] += v
		else:
			self[k] = v


	def __add__(self, other):
		tmp = self.copy()
		if isinstance(other, self.SCALARS):
			for k in tmp.keys():
				tmp[k] += other
		else:
			for (k, v) in other.items():
				try:
					tmp[k] += v
				except KeyError:
					tmp[k] = v
		return tmp


	def __gt__(self, other):
		tmp = dict_vector()
		if isinstance(other, self.SCALARS):
			for (k, v) in self.items():
				tmp[k] = v > other
		else:
			for (k, v) in self.items():
				tmp[k] = v > other.get(k, self.ZERO)
			for (k, v) in other.items():
				if k not in tmp:
					tmp[k] = self.get(k, self.ZERO) > v
		return tmp


	def __lt__(self, other):
		tmp = dict_vector()
		if isinstance(other, self.SCALARS):
			for (k, v) in self.items():
				tmp[k] = v < other
		else:
			for (k, v) in self.items():
				tmp[k] = v < other.get(k, self.ZERO)
			for (k, v) in other.items():
				if k not in tmp:
					tmp[k] = self.get(k, self.ZERO) < v
		return tmp


	def __ge__(self, other):
		tmp = dict_vector()
		if isinstance(other, self.SCALARS):
			for (k, v) in self.items():
				tmp[k] = v >= other
		else:
			for (k, v) in self.items():
				tmp[k] = v >= other.get(k, self.ZERO)
			for (k, v) in other.items():
				if k not in tmp:
					tmp[k] = self.get(k, self.ZERO) >= v
		return tmp


	def __le__(self, other):
		tmp = dict_vector()
		if isinstance(other, self.SCALARS):
			for (k, v) in self.items():
				tmp[k] = v <= other
		else:
			for (k, v) in self.items():
				tmp[k] = v <= other.get(k, self.ZERO)
			for (k, v) in other.items():
				if k not in tmp:
					tmp[k] = self.get(k, self.ZERO) <= v
		return tmp




	def __mul__(self, other):
		tmp = dict_vector()
		if isinstance(other, self.SCALARS):
			tmp = self.copy()
			for (k, v) in self.items():
				tmp[k] = v * other
		else:
			for (k, v) in other.items():
				tmp[k] = self.get(k, self.ZERO) * v
			for (k, v) in self.items():
				if k not in tmp:
					tmp[k] = v * other.get(k, self.ZERO)
		return tmp



	def __div__(self, other):
		if isinstance(other, self.SCALARS):
			if other == self.ZERO:
				raise ZeroDivisionError, "float denominator"
			else:
				tmp = dict_vector()
				for (k, v) in self.items():
					tmp[k] = v / other
		else:
			tmp = dict_vector()
			for (k, v) in other.items():
				if v != self.ZERO:
					tmp[k] = self.get(k, self.ZERO) / v
				else:
					raise ZeroDivisionError, "denom[%s]=%s" % (str(k), str(v))
			for (k, v) in self.items():
				if k not in tmp:
					try:
						tmp[k] = v / other[k]
					except KeyError:
						raise ZeroDivisionError, "denom[%s] is empty" % str(k)
		return tmp


	def __iadd__(self, other):
		if isinstance(other, self.SCALARS):
			for k in self.keys():
				self[k] += other
		else:
			for (k, v) in other.items():
				try:
					self[k] += v
				except KeyError:
					self[k] = v
		return self


	def __isub__(self, other):
		if isinstance(other, self.SCALARS):
			for k in self.keys():
				self[k] -= other
		else:
			for (k, v) in other.items():
				try:
					self[k] -= v
				except KeyError:
					self[k] = -v
		return self


	def __sub__(self, other):
		tmp = self.copy()
		if isinstance(other, self.SCALARS):
			for k in tmp.keys():
				tmp[k] -= other
		else:
			for (k, v) in other.items():
				try:
					tmp[k] -= v
				except KeyError:
					tmp[k] = -v
		return tmp


	__radd__ = __add__
	__rmul__ = __mul__


	def __rsub__(self, other):
		tmp = self.copy()
		if isinstance(other, self.SCALARS):
			for k in tmp.keys():
				tmp[k] = other - tmp[k]
		else:
			for (k, v) in other.items():
				try:
					tmp[k] = v - tmp[k]
				except KeyError:
					tmp[k] = v
		return tmp

	
	def __abs__(self):
		tmp = dict_vector()
		for (k, v) in self.items():
			tmp[k] = __builtin__.abs(v)
		return tmp



	def __invert__(self):
		tmp = dict_vector()
		for (k, v) in self.items():
			tmp[k] = -v
		return tmp


	def sum(self):
		return reduce(operator.add, self.values())


	def median(self):
		return gpkmisc.median(self.values())


	def all(self):
		for v in self.values():
			if not v:
				return False
		return True


	def float(self):
		tmp = dict_vector()
		for (k, v) in self.items():
			tmp[k] = float(v)
		return tmp


	def int(self):
		tmp = dict_vector()
		for (k, v) in self.items():
			tmp[k] = int(v)
		return tmp


	def iint(self):
		for k in self.keys():
			self[k] = int(tmp[v])


	def ifloat(self):
		for k in self.keys():
			self[k] = float(tmp[v])


	def round(self):
		tmp = dict_vector()
		for (k, v) in self.items():
			tmp[k] = __builtin__.round(v)
		return tmp


	def iround(self):
		for k in self.keys():
			self[k] = __builtin__.round(tmp[v])


	def idropzero(self):
		drops = []
		for (k, v) in self.items():
			if v == self.ZERO:
				drops.append(k)
		for k in drops:
			del self[k]

	def dropzero(self):
		tmp = dict_vector()
		for (k, v) in self.items():
			if not v == self.ZERO:
				tmp[k] = v
		return tmp

def round(x):
	return x.round()


def min_within(x):
	best = None
	vmin = None
	first = True
	for (k, v) in x.items():
		if first or v < vmin:
			vmin = v
			best = k
			first = False
	if first:
		raise ValueError, "Empty input sequence"
	return (best, vmin)


def max_within(x):
	best = None
	vmin = None
	first = True
	for (k, v) in x.items():
		if first or v > vmin:
			vmin = v
			best = k
			first = False
	if first:
		raise ValueError, "Empty input sequence"
	return (best, vmin)


def min_between(*x):
	tmp = dictops.dict_of_lists()
	scalars = []
	for xx in x:
		if isinstance(xx, (int, float)):
			scalars.append(xx)
		else:
			for (k, v) in xx.items():
				tmp.add(k, v)
	rv = dict_vector()
	for (k, vlist) in tmp.items():
		rv[k] = min(vlist + scalars)
	return rv



"""Fit a linear transform to a bunch of tt
input/output vectors."""


from gmisclib import Num
from gmisclib import gpk_lsq
from gmisclib import gpk_rlsq


def err_before_fit(data, minsv=None, minsvr=None):
	"""How much variation did the data have before the fit?
	This is used to compare with the error after the fit, to
	allow a F-test or ANOVA.    Strictly speaking, we compute
	the mean-squared error about a constant.
	@return: Returns the per-coordinate sum-squared-error
		of the output coordinates.
	@rtype: C{numpy.ndarray}
	@param data: See L{pack}.
	@type data: See L{pack}.
	"""

	tmp = []
	for (ioc) in data:
		if len(ioc) == 2:
			ic, oc = ioc
			tmp.append( (Num.zeros((0,), Num.Float), oc) )
		elif len(ioc) == 3:
			ic, oc, w = ioc
			w = float(w)
			assert w >= 0.0
			tmp.append( (Num.zeros((0,), Num.Float), oc, w) )
	A, B, errs, sv, ranks = localfit(tmp, minsv=minsv, minsvr=minsvr, constant=True)
	return errs



def pack(data, constant=True):
	"""Prepare the data array and (optionally) weight the data.
	@param data: [(input_coordinates, output_coordinates), ...]
		or [(input_coordinates, output_coordinates, weight), ...].
		A list of (in,out) tuples corresponding to the
		independent and dependent parameters of the linear transform.
		Both C{in} and C{out} are one-dimensional
		numpy vectors.   They don't need to have
		the same length, though (obviously) all instances of "in"
		need to have the same length and all instances of "out"
		also need to match one another.
		If C{weight} is given, it must be a scalar.
	@type data: [(L{numpy.ndarray}, L{numpy.ndarray}), ...] or
		[(L{numpy.ndarray}, L{numpy.ndarray}, float), ...]
	"""
	ic, oc = data[0][:2]	# We ignore the weight, if any.
	#: m will be the number of output coordinates (data points).
	m = oc.shape[0]
	#: n will be the number of input coordinates (parameters).
	n = ic.shape[0]
	#: Add a constant term.
	use_c = bool(constant)
	nd = len(data)
	i = Num.zeros((nd, n+use_c), Num.Float)
	o = Num.zeros((nd, m), Num.Float)
	for (j, iow) in enumerate(data):
		assert isinstance(iow, tuple)
		if len(iow) == 2:
			w = 1.0
			ic, oc = iow
		elif len(iow) == 3:
			ic, oc, w = iow
			w = float(w)
			assert w >= 0.0
		else:
			raise ValueError, "Needs 2-tuple (i,o) or 3-tuple (i,o,w)"
		o[j,:] = oc*w
		if use_c:
			i[j,0] = 1.0
			i[j,1:] = ic
		else:
			i[j,:] = ic
		Num.multiply(i[j,:], w, i[j,:])
	return (i, o, m, n)




# This is used to compute the matrix of derivitives,
# which is then used to compute the step in mcmcSQ.py
def localfit(data, minsv=None, minsvr=None, constant=True):
	"""Does a linear fit to data via a singular value decomposition
	algorithm.
	It returns the matrix A and vector B such that
	C{A*input_coordinates+B} is the best fit to C{output_coordinates}.
	@param minsv: sets the minimum useable singular value;
	@type minsv: float or None
	@param minsvr: sets the minimum useable s.v. in terms of the largest s.v.
	@type minsvr: float or None
	@param constant: Do you want a constant built into the linear equations?
	@type constant: Boolean
	@return: (A, B, errs, sv, ranks) where
		- A is a 2-D C{numpy.ndarray} matrix.
		- B is a 1-D C{numpy.ndarray} vector.
		- errs is a C{numpy.ndarray} vector, one value for each output coordinate.
			It gives the sum of squared residuals.
		- sv are the singular values, sorted into decreasing order.
	@param data: list of tuples.   See L{pack}.
	@type data: See L{pack}.
	"""

	ii, oo, m, n = pack(data, constant=constant)
	data = None	# Reclaim memory
	soln = gpk_lsq.linear_least_squares(ii, oo, minsv=minsv, minsvr=minsvr)
	ii = None	# Free unneeded memory.
	oo = None	# Free unneeded memory.
	errs = Num.sum( soln.residual()**2, axis=0 )
	sv = soln.sv()
	x = soln.x()
	rank = soln.rank()
	assert sv.shape[0]>=rank and sv.shape[0]<=n+1, "sv.shape=%s rank=%d m=%d n=%d" % (str(sv.shape), rank, m, n)
	assert rank <= 1+n
	assert len(errs) == m
	assert len(x.shape) == 2
	if constant:
		return ( Num.transpose(x[1:,:]), x[0,:],
			errs, Num.sort(sv)[::-1], rank
			)
	return ( Num.transpose(x), None,
			errs, Num.sort(sv)[::-1], rank
			)


# This is used to compute the matrix of derivitives,
# which is then used to compute the step in mcmcSQ.py
def r_localfit(data, minsv=None, minsvr=None):
	"""Data is [ (input_coordinates, output_coordinates), ... ]
	Minsv sets the minimum useable singular value;
	minsvr sets the minimum useable s.v. in terms of the largest s.v..
	It returns the matrix A and vector B such that the best fit
	is A*input_coordinates+B in a tuple
	(A, B, errs, ranks).
	errs is a vector, one value for each output coordinate.
	Rank is the minimum rank of the various fits.

	Warning! Not tested.
	"""

	ii, oo, m, n = pack(data)
	data = None	# Reclaim memory
	errs = []
	rank = None
	const = Num.zeros((m,), Num.Float)
	coef = Num.zeros((n,m), Num.Float)
	for j in range(m):
		assert oo.shape[1] == m
		soln = gpk_rlsq.robust_linear_fit(ii, oo[:,j], 1, minsv=minsv, minsvr=minsvr)
		errs.append( Num.sum( soln.residual()**2 ) )
		if rank is None or rank > soln.rank():
			rank = soln.rank()
		const[j] = soln.x()[0]
		coef[:,j] = soln.x()[1:]
	ii = None	# Free unneeded memory.
	oo = None	# Free unneeded memory.
	assert rank <= 1+n
	assert len(errs) == m
	return ( Num.transpose(coef), const, errs, rank)



def fit_giving_sigmas(data, minsv=None, minsvr=None, constant=True):
	"""Does a linear fit to data via a singular value decomposition
	algorithm.
	It returns the matrix A and vector B such that
	C{A*input_coordinates+B} is the best fit to C{output_coordinates}.
	@param minsv: sets the minimum useable singular value;
	@param minsvr: sets the minimum useable s.v. in terms of the largest s.v.
	@type minsv: float or None
	@type minsvr: float or None
	@param constant: Do you want a constant built into the linear equations?
	@type constant: Boolean
	@return: (A, B, errs, sv, ranks) where
		- A is a 2-D L{numpy.ndarray} matrix.
		- B is a 1-D L{numpy.ndarray} vector.
		- errs is a vector, one value for each output coordinate.
		- sv are the singular values, sorted into decreasing order.
	@param data: [(input_coordinates, output_coordinates), ...].
		A list of (in,out) tuples corresponding to the
		independent and dependent parameters of the linear transform.
		Both C{in} and C{out} are one-dimensional
		L{numpy vectors<numpy.ndarray>}.   They don't need to have
		the same length, though (obviously) all instances of "in"
		need to have the same length and all instances of "out"
		also need to match one another.
	@type data: [(L{numpy.ndarray}, L{numpy.ndarray}), ...]
	"""

	ii, oo, m, n = pack(data, constant=constant)
	data = None	# Reclaim memory
	soln = gpk_lsq.linear_least_squares(ii, oo, minsv=minsv, minsvr=minsvr)
	ii = None	# Free unneeded memory.
	oo = None	# Free unneeded memory.
	x = soln.x()
	sigmas = Num.sqrt(soln.x_variances())
	assert sigmas.shape == x.shape
	assert len(x.shape) == 2
	if constant:
		return ( Num.transpose(x[1:,:]), x[0,:],
			 Num.transpose(sigmas[1:,:]), sigmas[0,:]
			)
	return ( Num.transpose(x), None,
			Num.transpose(sigmas), None
			)


def leaktest():
	import RandomArray
	while 1:
		d = []
		for i in range(100):
			d.append( (RandomArray.standard_normal((30,)),
					RandomArray.standard_normal((1000,))))
		localfit(d)

def test_localfit11(r):
	d = [	(Num.array((0,)), Num.array((-1,))),
		(Num.array((1,)), Num.array((0,))),
		(Num.array((2,)), Num.array((1+1e-7,))),
		(Num.array((3,)), Num.array((2,))),
		]
	if r:
		# INCOMPATIBILITY HERE!
		coef, const, errs, sv, rank = r_localfit(d)
	else:
		coef, const, errs, sv, rank = localfit(d)
	assert const.shape == (1,)
	assert abs(const[0]-(-1)) < 1e-6
	assert coef.shape == (1,1)
	assert abs(coef[0,0]-1) < 1e-6
	assert len(errs)==1
	assert errs[0] < 1e-6
	assert len(sv) == 2
	assert rank == 2
	assert Num.alltrue(err_before_fit(d) >= errs)


def test_localfit11e():
	d = [	(Num.array((0.0,)), Num.array((0.0,))),
		(Num.array((1.0,)), Num.array((1.0,))),
		(Num.array((2.0,)), Num.array((0.0,))),
		(Num.array((3.0,)), Num.array((-2.0,))),
		(Num.array((4.0,)), Num.array((0.0,))),
		(Num.array((5.0,)), Num.array((1.0,))),
		(Num.array((6.0,)), Num.array((0.0,))),
		]
	coef, const, errs, sv, rank = localfit(d)
	print 'coef=', coef
	print 'const=', const
	print 'errs=', errs
	assert const.shape == (1,)
	assert abs(const[0]-0) < 1e-6
	assert coef.shape == (1,1)
	assert abs(coef[0,0]-0) < 1e-6
	assert len(errs)==1
	assert abs(errs[0]-6) < 1e-6
	assert len(sv) == 2
	assert rank == 2
	assert Num.sum(Num.absolute(err_before_fit(d)-errs)) < 1e-6


def test_localfit21():
	d = [ (Num.array((0,0)), Num.array((-1,))),
		(Num.array((1,2)), Num.array((0,))),
		(Num.array((2,-1)), Num.array((1,))) ]
	coef, const, errs, sv, rank = localfit(d)
	assert const.shape == (1,)
	assert abs(const[0]-(-1)) < 1e-6
	assert coef.shape == (1,2)
	assert Num.sum(Num.absolute(coef-[[1,0]])) < 1e-6
	assert len(errs)==1
	assert errs[0] < 1e-6
	assert len(sv) == 3
	assert rank == 3
	assert Num.alltrue(err_before_fit(d) >= errs)

def test_localfit21u():
	d = [ (Num.array((0,0)), Num.array((-1,))),
		(Num.array((2,-1)), Num.array((1,))) ]
	coef, const, errs, sv, rank = localfit(d)
	assert const.shape == (1,)
	assert abs(const[0]-(-1)) < 1e-6
	assert abs(const[0] + 2*coef[0,0]-coef[0,1] - 1) < 1e-6
	assert coef.shape == (1,2)
	assert len(errs)==1
	assert errs[0] < 1e-6
	assert len(sv) == 2
	assert rank == 2
	assert Num.alltrue(err_before_fit(d) >= errs)

def test_localfit22(r):
	d = [ (Num.array((0,0)), Num.array((-1,0))),
		(Num.array((1,2)), Num.array((0,1.0))),
		(Num.array((-1,2)), Num.array((-2,1.0))),
		(Num.array((-1,1)), Num.array((-2+1e-7,0.5+1e-7))),
		(Num.array((2,-1)), Num.array((1,-0.5))) ]

	if r:
		coef, const, errs, sv, rank = r_localfit(d)
	else:
		coef, const, errs, sv, rank = localfit(d)

	print const
	print coef
	print  errs
	print  sv
	print rank
	assert const.shape == (2,)
	assert Num.sum(Num.absolute(const - [-1,0])) < 1e-6
	assert coef.shape == (2,2)
	assert Num.sum(Num.absolute(coef[0]-[1,0])) < 1e-6
	assert Num.sum(Num.absolute(coef[1]-[0,0.5])) < 1e-6
	assert len(errs)==2
	assert Num.sum(errs) < 1e-6
	assert len(sv) == 3
	assert rank == 3
	assert Num.alltrue(err_before_fit(d) >= errs)


def test_fgs1():
	N = 2000
	DSIG = 10.0
	import random
	import math
	d = []
	for i in range(N):
		s = random.normalvariate(0.0, DSIG)
		d.append((Num.array([-1.0]), Num.array([1-s])))
		d.append((Num.array([1.0]), Num.array([1+s])))
	coef, const, scoef, sconst = fit_giving_sigmas(d)
	assert abs(coef[0][0]) < 3*DSIG/math.sqrt(N)
	assert abs(coef[0][0]) > 0.001*DSIG/math.sqrt(N)
	assert abs(const[0]-1.0) < DSIG/math.sqrt(N)
	assert sconst[0] < DSIG/math.sqrt(N)
	assert abs(coef[0][0]) < 4*DSIG/2.0/math.sqrt(N)
	assert scoef[0][0] < 1.3*DSIG/math.sqrt(N)
	assert scoef[0][0] > 0.7*DSIG/math.sqrt(N)
	print 'FGS1 OK'


def test_wt():
	N = 1000
	DSIG = 1.0
	import random
	d = []
	for i in range(N):
		s = random.normalvariate(0.0, DSIG)
		x = random.normalvariate(0.0, DSIG)
		w = random.expovariate(1.0)
		d.append((Num.array([-x]), Num.array([1-s]), w))
		d.append((Num.array([x]), Num.array([1+s]), w))
		d.append((Num.array([x]), Num.array([1-s]), w))
		d.append((Num.array([-x]), Num.array([1+s]), w))
	var1 = err_before_fit(d)
	A, B, errs, sv, ranks = localfit(d)
	assert Num.absolute((var1-errs)).sum() < 0.1


if __name__ == '__main__':
	test_wt()
	test_localfit11(False)
	test_localfit11e()
	test_localfit21()
	test_localfit21u()
	for r in [False]:	# Do Not test r_localfit()
		test_localfit11(r)
		test_localfit22(r)
	test_fgs1()



"""This is a helper module to make use of mcmc.py and mcmc_big.py.
It allows you to conveniently run a Monte-Carlo simulation of any
kind until it converges (L{stepper.run_to_bottom}) or until it
has explored a large chunk of parameter space (L{stepper.run_to_ergodic}).

It also helps you with logging the process.

When run in parallel, each processor does its thing more-or-less
independently.   However, every few steps, they exchange notes on
their current progress.   If one finds an improved vertex, it will be
passed on to other processors via MPI.

Tested with mpi4py version 1.1.0, http://mpi4py.scipy.org
"""

import sys
from mpi4py import MPI	# This uses mpi4py
import random
import numpy
from gmisclib import die
from gmisclib import mcmc
from gmisclib import mcmc_helper as MCH
Debug = 0
from gmisclib.mcmc_helper import TooManyLoops, warnevery, logger_template, test

mpi = MPI.COMM_WORLD

class _counter(object):
	def __init__(self):
		self.count = 0

	def update(self, accepted):
		self.count += accepted

	def get(self):
		return self.count



class _accum_erg(object):
	def __init__(self):
		self.erg = 0.0
		self.last_resetid = -1

	def update(self, x):
		"""Increment an accumulator, but reset it to
		zero if the underlying stepper has called reset().
		The increment is by the ergodicity estimate, so
		C{self.erg} should approximate how many times the
		MCMC process had wandered across the probability
		distribution.
		"""
		tmp = x.reset_id()
		if tmp != self.last_resetid:
			self.erg = 0.0
			self.last_resetid = tmp
		else:
			self.erg += x.ergodic()


	def get(self):
		return self.erg



class stepper(MCH.stepper):
	def __init__(self, x, maxloops=-1, logger=None):
		die.info('# mpi stepper rank=%d size=%d' % (rank(), size()))
		assert maxloops == -1
		self.maxloops = -1
		MCH.stepper.__init__(self, x, maxloops, logger)


	def reset_loops(self, maxloops=-1):
		assert maxloops == -1
		self.maxloops = -1


	def communicate_hook(self, id):
		self.note('chook iter=%d' % self.iter, 4)
		if size() > 1:
			self.note('chook active iter=%d' % self.iter, 3)
			c = self.x.current()
			v = c.vec()
			lp = c.logp()

			r = rank()
			s = size()
			self.note('sendrecv from %d to %d' % (r, (r+1)%s), 5)
			nv, nlp, nid = mpi.sendrecv(sendobj=(v, lp, id),
								dest=(r+1)%s, sendtag=self.MPIID,
								source=(r+s-1)%s,
								recvtag=self.MPIID
								)

			self.note('communicate succeeded from %s' % nid, 1)
			delta = nlp - lp
			if self.x.acceptable(delta):
				q = self.x._set_current(c.new(nv-v, logp=nlp))
				self.note('communicate accepted: %s' % q, 1)
			else:
				self.note('communicate not accepted %g vs %g' % (nlp, lp), 1)
				self.x._set_current(self.x.current())
			sys.stdout.flush()


	MPIID = 1241

	# def _not_enough_changes(self, n, ncw):
		# mytmp = n < ncw
		# self.note('_nyc pre')
		# ntrue = mpi.allreduce(int(mytmp), MPI.SUM)
		# self.note('_nyc post')
		# return ntrue*3 >= size()


	def _nc_get_hook(self, nc):
		self.note('_nye pre', 5)
		ncsum = mpi.allreduce(float(nc), MPI.SUM)
		self.note('_nye post', 5)
		return ncsum/float(size())


	def _not_at_bottom(self, xchanged, nchg, es, dotchanged, ndot):
		mytmp = (numpy.sometrue(numpy.less(xchanged,nchg))
					or es<1.0 or dotchanged<ndot
					or self.x.acceptable.T()>1.5
				)
		self.note('_nab pre', 5)
		ntrue = mpi.allreduce(int(mytmp), MPI.SUM)
		self.note('_nab post', 5)
		return ntrue*4 >= size()


	def join(self, id):
		self.note('pre join %s' % id, 5)
		rootid = mpi.bcast(id)
		assert rootid == id
		self.note('post join %s' % id, 5)


	def note(self, s, lvl):
		if Debug >= lvl:
			sys.stderr.writelines('# %s, rank=%d\n' % (s, rank()))
			sys.stderr.flush()


def precompute_logp(lop):
	"""Does a parallel evaluation of logp for all items in lop.
	"""
	nper = len(lop)//size()
	r = rank()
	mychunk = lop[r*nper:(r+1)*nper]
	for p in mychunk:
		q = p.logp()
		print 'logp=', q, 'for rank', r
	for r in range(size()):
		nc = mpi.bcast(mychunk, r)
		lop[r*nper:(r+1)*nper] = nc
	mpi.barrier()


def is_root():
	return mpi.Get_rank() == 0

def size():
	return mpi.Get_size()

def rank():
	return mpi.Get_rank()


def test():
	def test_logp(x, c):
		# print '#', x[0], x[1]
		return -(x[0]-x[1]**2)**2 + 0.001*x[1]**2
	x = mcmc.bootstepper(test_logp, numpy.array([0.0,2.0]),
				numpy.array([[1.0,0],[0,2.0]]))
	print 'TEST: rank=', rank()
	thr = stepper(x)
	# nsteps = thr.run_to_bottom(x)
	# print '#nsteps', nsteps
	# assert nsteps < 100
	for i in range(2):
		print 'RTC'
		thr.run_to_change(2)
		print 'RTE'
		thr.run_to_ergodic(1.0)
		print 'DONE'
	thr.close()


if __name__ == '__main__':
	test()




"""Levenshtein distance"""

import numpy

def dist(s, t, csub, cinsert, cdel):
	"""Cost for converting string s into string t,
	@param s: starting string
	@param t: ending string
	@type s: string or array
	@type t: string or array
	@param csub: cost of converting a to b
	@type csub: dict((a,b) : float)
	@param cinsert: cost of insertion
	@type csub: dict(a: float)
	@param cdel: cost of deletion
	@type cdel: dict(a: float)
	"""
	def f(a, b):
		if a is None:
			return cinsert[b]
		elif b is None:
			return cdel[a]
		return csub[(a, b)]
	return distf(s, t, f)


def distf1(s, t, csub):
	"""Cost for converting string s into string t,
	@param s: starting string
	@param t: ending string
	@type s: string or array
	@type t: string or array
	@param csub: cost of converting a to b
	@type csub: function(a,b) : float
	"""
	m = len(s)
	n = len(t)
	d = numpy.zeros((m+1, n+1), numpy.float)
	for i in range(1, m+1):
		d[i,0] = d[i-1,0] + csub(s[i-1], None)
	for j in range(1, n+1):
		d[0,j] = d[0,j-1] + csub(None, t[j-1])
	
	for j in range(1, n+1):
		for i in range(1, m+1):
			d[i,j] = min(d[i-1,j] + csub(s[i-1], None),  # insertion
					d[i,j-1] + csub(None, t[j-1]),	# deletion
					d[i-1, j-1] + csub(s[i-1], t[j-1])	# sub
					)
	return d[m, n]


def distf2(s, t, csub):
	"""Cost for converting string s into string t,
	@param s: starting string
	@param t: ending string
	@type s: string or array
	@type t: string or array
	@param csub: cost of converting a to b
	@type csub: function(a,b) : float
	"""
	m = len(s)
	n = len(t)
	dj = numpy.zeros((m+1,))
	djm1 = numpy.zeros((m+1,))
	for i in range(1, m+1):
		dj[i] = dj[i-1] + csub(s[i-1], None)
	
	for j in range(1, n+1):
		tmp = djm1
		djm1 = dj
		dj = tmp
		dj[:] = 0.0
		dj[0] = djm1[0] + csub(None, t[j-1])
		for i in range(1, m+1):
			dj[i] = min(dj[i-1] + csub(s[i-1], None),  # insertion
					djm1[i] + csub(None, t[j-1]),	# deletion
					djm1[i-1] + csub(s[i-1], t[j-1])	# sub
					)
	return dj[m]


distf = distf2


def def_cost(a, b):
	if a is None or b is None:
		return 1
	if a==b:
		return 0
	return 1


def free_sub(a, b):
	if a is None or b is None:
		return 1
	return 0
	

def free_del(a, b):
	if b is None:
		return 0
	if a is None:
		return 1
	if a==b:
		return 0
	return 1


class text_cost(object):
	"""This is useful for differencing documents that have been parsed
	into lists of strings.
	"""

	def __init__(self, costfac=None, cachesize=None, word_cost=None, frac_exp=0.0):
		self.costfac = costfac
		self.cache = {}
		if cachesize is None:
			self.cachesize = 100000
		else:
			assert cachesize >= 0
			self.cachesize = cachesize

		if word_cost is None:
			self.word_cost = def_cost
		else:
			self.word_cost = word_cost
		self.frac_exp = float(frac_exp)



	def __call__(self, a, b):
		"""To be used by L{distf}.
		@type a: str
		@type b: str
		@return: a cost
		@rtype: presumably a float
		"""
		if a is None:
			a = ''
		else:
			if a == b:
				return 0.0
		if b is None:
			b = ''

		try:
			return self.cache[(a,b)]
		except KeyError:
			pass

		cost = distf(a, b, self.word_cost)
		cost /= float(max(len(a), len(b)))**self.frac_exp
		if len(self.cache) > self.cachesize:
			self.cache.pop()
		if self.costfac is not None:
			cost *= self.costfac(a, b)
		self.cache[(a,b)] = cost
		return cost



def test():
	assert distf("hello", "helpo", def_cost) == 1
	assert distf("hello", "helo", def_cost) == 1
	assert distf("hello", "hellox", def_cost) == 1
	assert distf("hello", "helloxx", def_cost) == 2
	assert distf("hello", "elo", def_cost) == 2
	assert distf("hello", "eelpo", def_cost) == 2
	assert distf("kitten", "sitting", def_cost) == 3
	assert distf("saturday", "sunday", def_cost) == 3
	assert distf("hello", "xxxxo", free_sub) == 0
	assert distf("hello", "xxxo", free_sub) == 1
	assert distf("hello", "xxo", free_sub) == 2
	assert distf("hello", "yxxxxo", free_sub) == 1
	assert distf("hello", "yxxxxoy", free_sub) == 2
	assert distf("hello", "hello", free_del) == 0
	assert distf("hello", "helo", free_del) == 0
	assert distf("hello", "hlo", free_del) == 0
	assert distf("helo", "hello", free_del) == 1
	assert distf("hlo", "hello", free_del) == 2
	assert distf("hello", "hallo", free_del) == 1
	assert distf("hlo", "hellp", free_del) == 3


def test2():
	a = ['Now', ' ', 'is', ' ', 'the', ' ', 'time', ' ', 'for', '\n', 'testing', '.']
	b = list(a)
	b[-1] = '!'
	assert distf(a, b, text_cost()) == 1
	b[-3] = '  '
	assert distf(a, b, text_cost()) == 3
	b = list(a)
	b[-4] = 'from'
	assert distf(a, b, text_cost()) == 2
	b[-4:-3] = []
	assert distf(a, b, text_cost()) == 3
	b = list(a)
	b[-4] = 'fir'
	assert abs( distf(a, b, text_cost(frac_exp=1.0)) - 1./3.) < 0.001


if __name__ == '__main__':
	test()
	test2()



#! python

from __future__ import with_statement

import sys
import Queue
import cPickle as CP
import threading
import subprocess
import StringIO

from gmisclib import die
from gmisclib import gpkmisc


class notComputed(object):
	"""Just a marker for values that haven't been computed."""
	pass


class LockedList(object):
	def __init__(self):
		self.lock = threading.Lock()
		self.data = []


	def put(self, i, x):
		with self.lock:
			if len(self.data) == i:
				self.data.append(x)
			elif len(self.data) > i:
				assert self.data[i] is notComputed
				self.data[i] = x
			else:
				while len(self.data) < i:
					# "self" is used as a marker for entries that
					# have not been set.   get() makes sure they
					# are all gone before returning an array.
					self.data.append(notComputed)
				self.data.append(x)
	
	def __len__(self):
		with self.lock:
			tmp = len(self.data)
		return tmp


	def get(self):
		tmp = []
		with self.lock:
			for x in self.data:
				if isinstance(x, raiseException):
					x.raise_me()
				assert x is not notComputed
				tmp.append(x)
		return tmp



class NoResponse(RuntimeError):
	def __init__(self, *s):
		RuntimeError.__init__(self, *s)


class RemoteException(Exception):
	def __init__(self, *s):
		Exception.__init__(self, *s)


class raiseException(object):
	def __init__(self, *s):
		self.info = s
		self.index = ""
		self.comment = ''

	def raise_me(self):
		raise RemoteException(*(self.info + (self.index,self.comment)) )



class ThreadJob(threading.Thread):
	def __init__(self, iqueue, olist, args, stdin, solock, verbose=False):
		threading.Thread.__init__(self)
		self.iqueue = iqueue
		self.olist = olist
		self.p = None
		if verbose:
			die.info("Spawning process: %s" % str(args))
		self.spawn_process(args)
		self.solock = solock
		for x in stdin:
			self.send(x)
	

	def run(self):
		# sys.stderr.write('run-starting\n')
		while True:
			# sys.stderr.write('run-loop\n')
			i, todo = self.iqueue.get()
			if i is None:
				self.iqueue.task_done()
				break
			# sys.stderr.write('run-loop %s %s\n' % (i, todo))
			try:
				self.send(todo)
			except IOError, x:
				die.warn("IO Error on send %d to worker: %s" % (i, str(x)))
				self.iqueue.put((i, todo))
				self.iqueue.task_done()
				break
			# sys.stderr.write('run-sent\n')
			q, so, se = self.get()
			# sys.stderr.write('run-got %s %s %s\n' % (q, so, se))
			self.olist.put(i, q)
			# sys.stderr.write('run-completed\n')
			with self.solock:
				if so:
					sys.stdout.writelines('#slot so%d ------\n' % i)
					sys.stdout.writelines(so)
					sys.stdout.flush()
				if se:
					sys.stderr.writelines('#slot se%d ------\n' % i)
					sys.stderr.writelines(se)
					sys.stderr.flush()
			# sys.stderr.write('run-bottom\n')
			self.iqueue.task_done()
			if isinstance(q, raiseException):
				die.warn("Exception from remote job (index=%d): %s" % (i, str(q)))
				q.index = "index=%d" % i
				q.comment = "stderr=%s" % gpkmisc.truncate('/'.join(se), 40)
				# Eat all remaining jobs
				while True:
					i, todo = self.iqueue.get()
					self.iqueue.task_done()
					if i is None:
						break
					self.olist.put(i, notComputed)


	def spawn_process(self, args):
		self.p = subprocess.Popen(args, stdin=subprocess.PIPE, stdout=subprocess.PIPE,
					close_fds=True
					)

	def send(self, todo):
		CP.dump(todo, self.p.stdin)
		self.p.stdin.flush()
		# sys.stdout.write('send - completed.\n')


	def get(self):
		while True:
			try:
				q, so, se = CP.load(self.p.stdout)
				return (q, so, se)
			except CP.UnpicklingError, y:
				die.warn("spread_load: Junk response: %s" % str(y))
				continue
		return None

	def close(self):
		self.p.stdin.close()
		self.p.wait()
		self.p.stdout.close()


def main(todo, list_of_args, stdin=None, verbose=False):
	"""Pass a bunch of work to other processes.
	@param stdin: a list of stuff to send to the other processes before the computation is
		properly commenced.
	@type stdin: list(whatever)
	@param todo: a list of work to do
	@type todo: list(whatever)
	@param list_of_args:
	@type list_of_args:  list(list(str))
	@rtype: list(whatever)
	@return: a list of the results produced by the other processes.
		Items in the returned list correspond to items in the todo list.
	"""
	# sys.stderr.write('main starting\n')
	iqueue = Queue.Queue()
	olist = LockedList()
	solock = threading.Lock()
	if stdin is None:
		stdin = []
	ths = []
	for args in list_of_args:
		t = ThreadJob(iqueue, olist, args, stdin, solock, verbose=verbose)
		# t.setDaemon(True)
		t.start()
		ths.append(t)
	# sys.stderr.write('all threads started\n')
	for (i, job) in enumerate(todo):
		# sys.stderr.write('put %d %s\n' % (i, job))
		iqueue.put( (i,job) )
	# sys.stderr.write('all work queued\n')
	for args in list_of_args:
		iqueue.put( (None,None) )
	iqueue.join()
	# sys.stderr.write('all threads joined\n')
	for t in ths:
		# sys.stderr.write('close %s\n' % t)
		t.close()
	# sys.stderr.write('main done\n')
	assert len(olist) == len(todo)
	return olist.get()



def test_worker():
	# sys.stderr.write('test_worker starting\n')
	while True:
		# sys.stderr.write('test_worker loop\n')
		try:
			control = CP.load(sys.stdin)
		except EOFError:
			# sys.stderr.write('test_worker break')
			break
		# sys.stderr.write('test_worker dump\n')
		CP.dump((control, ['stdout\n'], ['stderr\n']), sys.stdout, CP.HIGHEST_PROTOCOL)
		sys.stdout.flush()

def test():
	x = ['a', 'b', 'c', 'd', 'e']
	args = ['python', sys.argv[0], 'worker']
	y = main(x, [args]*3 )
	assert x == y



class unpickled_pseudofile(StringIO.StringIO):
	"""For testing.
	"""

	def __init__(self):
		StringIO.StringIO.__init__(self)

	def close(self):
		self.seek(0, 0)
		while True:
			try:
				d, so, se = CP.load(self)
			except EOFError:
				break
			sys.stdout.write('STDOUT:\n')
			sys.stdout.writelines(so)
			sys.stdout.write('STDERR:\n')
			sys.stdout.writelines(se)
			sys.stdout.write('d=%s\n' % str(d))
			sys.stdout.flush()


def one_shot_test(input):
	stdin = StringIO.StringIO()
	CP.dump(input, stdin)
	stdin.flush()
	stdin.seek(0, 0)
	stdout = unpickled_pseudofile()
	return (stdin, stdout)


if __name__ == '__main__':
	if len(sys.argv)==2 and sys.argv[1] == 'worker':
		test_worker()
	else:
		test()




import numpy
import math


def add_overlap(a, astart, b, bstart):
	"""Add arrays a and b in the overlap region.
	Return (data, start).
	If a, b are time series, they are assumed to have the
	same sampling rate.
	Astart and Bstart apply to the zeroth index.
	All other indices are assumed to match start and length.
	"""
	start = max(astart, bstart)
	end = min(astart+a.shape[0], bstart+b.shape[0])

	out = a[start-astart:end-astart] + b[start-bstart:end-bstart]

	return (out, start)




# def asinh(x):
        # sign = numpy.sign(x)
	# return sign*numpy.log(sign*x + numpy.sqrt(x*x + 1))
asinh = numpy.arcsinh

# def acosh(x):
	# assert numpy.all(numpy.less_equal(x, 1))
	# return numpy.log(x - numpy.sqrt(x*x-1))
acosh = numpy.arccosh


def zero_pad_end(d, padfactor=1):
	if padfactor == 0:
		return numpy.array(d, copy=True)
	elif padfactor<=0:
		raise ValueError, 'pad factor <= 0'
	assert len(d.shape) == 1
	n = d.shape[0]
	npad = int(round(n*padfactor))
	return numpy.concatenate( (d, numpy.zeros((npad,),  numpy.double)) )


def Poisson(nbar):
	"""Return a Poisson random integer, whose distribution has
	a mean = nbar.
	"""
	import random
	assert nbar >= 0.0
	if nbar < 20.0:
		L = math.exp(-nbar)
		k = -1
		p = 1.0
		while p >= L:
			k += 1
			p *= random.random()
	else:
		lp = 0.0
		lL = -nbar
		k = 0
		chunk = min( int(round (1 + nbar + 3*math.sqrt(nbar) )), 10000)
		while lp >= lL:
			ptmp = numpy.log(numpy.random.uniform(0.0, 1.0, chunk) )
			lpp = numpy.add.accumulate(ptmp) + lp
			fsm = numpy.nonzero(numpy.less(lpp, lL))[0]
			if fsm.shape[0] != 0:
				k += fsm[0]
				break
			k += ptmp.shape[0]
			lp = lpp[-1]
	return k



def _test_Poisson():
	Nbar = 0.01
	N = int(round(1000/Nbar))
	s = 0
	for i in range(N):
		s += Poisson(Nbar)
	assert abs(s-N*Nbar) < 5*math.sqrt(N*Nbar)


	Nbar = 25.0
	N = 1000
	s = 0
	for i in range(N):
		s += Poisson(Nbar)
	assert abs(s-N*Nbar) < 5*math.sqrt(N*Nbar)



def bevel_concat(a, b, bevel=0, bevel_overlap=1.0, delay=0,
			ta=None, tb=None):
	"""Concatenate two time series.  Bevel the edges,
	and overlap them slightly.

	Bevel_overlap controls the fractional overlap of the two bevels,
	and delay specifies an extra delay for b.

	If ta and/or tb are specified, return a tuple of
	(concatenated_time_series, tma, tmb) where tma and tmb are the locations
	corresponding to ta and tb in the corresponding input arrays.
	"""
	assert bevel >= 0
	if bevel > 0:
		bev = (0.5 + numpy.arange(bevel))/float(bevel)
	else:
		bev = 1
	bc = numpy.array(b, copy=True)
	bev = (0.5 + numpy.arange(bevel))/float(bevel)
	ans = a.shape[0]-1
	bns = bc.shape[0]-1
	bos = a.shape[0] - int(round(bevel_overlap*bevel)) + delay
	boe = bos + bc.shape[0]
	o = numpy.zeros((boe,), numpy.double)
	o[:a.shape[0]] = a
	if bevel > 0:
		numpy.multiply(o[:bevel], bev, o[:bevel])
		numpy.multiply(o[ans:ans-bevel:-1], bev, o[ans:ans-bevel:-1])
		numpy.multiply(bc[:bevel], bev, bc[:bevel])
		numpy.multiply(bc[bns:bns-bevel:-1], bev, bc[bns:bns-bevel:-1])
	numpy.add(o[bos:boe], bc, o[bos:boe])
	if ta is not None or tb is not None:
		if tb is not None:
			tb += bos
		return (o, ta, tb)
	else:
		return o


def argmax(a):
	i = numpy.argmax(a, axis=None)
	o = []
	for n in reversed(a.shape):
		o.append( i % n )
		i = i//n
	o.reverse()
	return tuple(o)


def _test_argmax():
	x = numpy.array([1,2,3,4,3])
	assert argmax(x) == (3,)
	x = numpy.array([[1,2,3,4,3], [0,1,0,1,0]])
	assert x[argmax(x)] == 4
	x = numpy.array([[1,2,3,4,3], [0,1,0,5,0]])
	assert x[argmax(x)] == 5
	x = numpy.array([[1,2], [3,4], [5,6], [0,1],[0,5],[0,7]])
	assert x[argmax(x)] == 7
	x = numpy.zeros((5,4,3,3,1,2), numpy.double)
	x[2,1,0,1,0,1] = 100.0
	assert argmax(x) == (2,1,0,1,0,1)
	x[0,0,0,0,0,0] = 200.0
	assert argmax(x) == (0,0,0,0,0,0)
	x[4,3,2,2,0,1] = 300.0
	assert argmax(x) == (4,3,2,2,0,1)
	x[4,3,2,2,0,0] = 400.0
	assert argmax(x) == (4,3,2,2,0,0)
	x[4,3,2,0,0,0] = 500.0
	assert argmax(x) == (4,3,2,0,0,0)


def N_maximum(a):
	assert len(a.shape) == 1
	return a[numpy.argmax(a)].item()


def N_minimum(a):
	assert len(a.shape) == 1
	return a[numpy.argmin(a)].item()




def N_frac_rank(a, fr):
	assert 0 <= fr <= 1.0
	tmp = numpy.sort(a)
	n = tmp.shape[0]
	assert n > 0, "Zero-sized array: cannot compute rank."
	return tmp[int(round((n-1)*fr))].item()


def N_mean_ad(a):
	"""Mean absolute deviation.   For a multi-dimensional array,
	it takes the MAD along the first axis, so
	N_mean_ad(x)[0]==N_mean_ad(x[:,0]).
	"""
	ctr = numpy.median(a)
	diff = numpy.subtract(a, ctr)
	return numpy.sum(numpy.absolute(diff), axis=0)/(diff.shape[0]-1)


def _test_N_mean_ad():
	x = numpy.zeros((2,1), numpy.double)
	x[0,0] = 1
	x[1,0] = 0
	assert numpy.allclose(N_mean_ad(x), [1.0])
	x = numpy.zeros((5,7), numpy.double)
	x[0,0] = 1
	y = N_mean_ad(x)
	assert y.shape == (7,)
	assert numpy.allclose(y, [(1.0-0.0)/4.0, 0, 0, 0, 0, 0, 0])
	assert abs(N_mean_ad(x)[0]-N_mean_ad(x[:,0])) < 0.001


def median_across(list_of_vec):
	"""Returns an element-by-element median of a list of Numeric vectors."""
	assert len(list_of_vec[0].shape) == 1
	o = numpy.zeros(list_of_vec[0].shape, numpy.double)
	tmp = numpy.zeros((len(list_of_vec),), numpy.double)
	for t in list_of_vec:
		assert t.shape == o.shape
	for i in range(o.shape[0]):
		for (j, v) in enumerate(list_of_vec):
			tmp[j] = v[i]
		o[i] = numpy.median(tmp)
	return o


# Median along the first axis.
def N_median(a, axis=0):
	"""Returns an element-by-element median of a list of Numeric vectors."""
	if len(a.shape) ==  1:
		return numpy.median(a)
	assert len(a.shape) == 2
	if axis == 1:
		o = numpy.zeros((a.shape[0],), numpy.double)
		for i in range(o.shape[0]):
			o[i] = numpy.median(a[i])
	o = numpy.zeros((a.shape[1],), numpy.double)
	a = numpy.array(a)
	for i in range(o.shape[0]):
		o[i] = numpy.median(a[:,i])
	return o

def N_median_across(a):
	return N_median(a, axis=0)


variance = numpy.var
stdev = numpy.std
make_diag = numpy.diag


def set_diag(x, a):
	"""Set the diagonal of a matrix x to be the vector a.
	If a is shorter than the diagonal of x, just set the beginning."""

	assert len(a.shape) == 1
	assert len(x.shape) == 2
	n = a.shape[0]
	assert x.shape[0] >= n
	assert x.shape[1] >= n
	for i in range(n):
		x[i,i] = a[i]


# Take the median along the first coordinate of a 2-D matrix,
# so if m.shape=(2,4) then the output will have shape (4,)
def _test_N_median_across():
	x = numpy.zeros((3,2), numpy.double)
	x[0,0] = 1
	x[1,0] = 2
	x[2,0] = 3
	y = N_median(x, axis=0)
	# y = N_median_across(x)
	print 'median_across=', y
	assert numpy.allclose(y, [2.0, 0.0])


def limit(low, x, high):
	return numpy.clip(x, low, high)



def trimmed_mean_sigma_across(list_of_vec, weights, clip):
	import gpkavg
	"""Returns an element-by-element trimmed_mean of a list of Numeric vectors.
	For instance, the first component of the output vector is the trimmed mean
	of the first component of all of the input vectors.
	"""
	import gpkavg
	assert len(list_of_vec[0].shape) == 1
	om = numpy.zeros(list_of_vec[0].shape, numpy.double)
	osig = numpy.zeros(list_of_vec[0].shape, numpy.double)
	tmp = numpy.zeros((len(list_of_vec),), numpy.double)
	for t in list_of_vec:
		assert t.shape == om.shape
	for i in range(om.shape[0]):
		for (j, v) in enumerate(list_of_vec):
			tmp[j] = v[i]
		om[i], osig[i] = gpkavg.avg(tmp, weights, clip)
	return (om, osig)

def trimmed_mean_across(list_of_vec, weights, clip):
	return trimmed_mean_sigma_across(list_of_vec, weights, clip)[0]

def trimmed_stdev_across(list_of_vec, weights, clip):
	return trimmed_mean_sigma_across(list_of_vec, weights, clip)[1]


def vec_variance(x):
	"""Take a component-by-component variance of a list of vectors."""
	n = len(x)
	if n < 2:
		raise ValueError, "Cannot take variance unless len(x)>1"
	x0 = x[0]
	sh = x0.shape
	s = numpy.zeros(sh, numpy.double)
	ss = numpy.zeros(sh, numpy.double)
	for xi in x:
		assert xi.shape == sh
		tmp = xi - x0
		numpy.add(s, tmp, s)
		numpy.add(ss, tmp**2, ss)
	return numpy.maximum(ss-s**2/n, 0.0)/(n-1)




def qform(vec, mat):
	"""A quadratic form: vec*mat*vec,
	or vecs*mat*transpose(vecs)"""
	if len(vec.shape) != 1 or len(mat.shape) != 2:
		raise ValueError, ': '.join(["Can't handle input",
					     "requires vector*matrix(vector)",
					     "shapes are %s and %s" % (
							vec.shape, mat.shape)
						])
	if mat.shape != (vec.shape[0], vec.shape[0]):
		raise ValueError, ': '.join([
					"Matrix must be square and match the length of vector",
					"shapes are %s and %s" % (vec.shape, mat.shape)
					])
	return numpy.dot(vec, numpy.dot(mat, vec))



def KolmogorovSmirnov(d1, d2, w1=None, w2=None):
	d1 = numpy.asarray(d1)
	d2 = numpy.asarray(d2)
	assert len(d1.shape) == 1, 'd1.shape=%s' % str(d1.shape)
	assert len(d2.shape) == 1, 'd2.shape=%s' % str(d2.shape)
	if w1 is None:
		w1 = numpy.ones(d1.shape, numpy.double)
	if w2 is None:
		w2 = numpy.ones(d2.shape, numpy.double)
	ws1 = numpy.sum(w1)
	ws2 = numpy.sum(w2)
	i1 = numpy.argsort(d1)
	i2 = numpy.argsort(d2)
	c1 = 0.0
	c2 = 0.0
	j1 = 0
	j2 = 0
	maxdiff = 0.0
	while True:
		if abs(c1-c2) > maxdiff:
			maxdiff = abs(c1 - c2)
		if j1 < i1.shape[0]-1 and j2<i2.shape[0]-1:
			if d1[i1[j1]] < d2[i2[j2]]:
				j1 += 1
				c1 += w1[i1[j1]]/ws1
			elif d1[i1[j1]] > d2[i2[j2]]:
				j2 += 1
				c2 += w2[i2[j2]]/ws2
			else:
				j1 += 1
				c1 += w1[i1[j1]]/ws1
				j2 += 1
				c2 += w2[i2[j2]]/ws2
		elif j1==i1.shape[0]-1 and j2==i2.shape[0]-1:
			break
		elif j1 < i1.shape[0]-1:
			j1 += 1
			c1 += w1[i1[j1]]/ws1
		else:
			j2 += 1
			c2 += w2[i2[j2]]/ws2
	return maxdiff


def _testKS():
	assert KolmogorovSmirnov([1, 2, 3.01, 3, 4, 5], [1, 2, 3.01, 3, 4, 5]) < 0.001
	assert abs( KolmogorovSmirnov([1, 2, 3.01, 3, 4, 5], [1, 2, 3, 4, 5, 6]) - 0.16667) < 0.001
	print KolmogorovSmirnov([1, 2, 3.01, 3, 4, 5], [1, 2, 3, 4, 5])




def interpN(a, t):
	"""Interpolate array a to floating point indices, t,
	via nearest-neighbor interpolation.
	Returns a Numeric array.
	"""
	ii = numpy.around(t)
	n = a.shape[0]
	if not numpy.alltrue((ii>=0) * (ii<=n-1)):
		raise IndexError, "Index out of range."
	iii = ii.astype(numpy.int)
	return numpy.take(a, iii, axis=0)



def interp(a, t):
	"""Interpolate to a specified time axis.
	This does a linear interpolation.
	A is a Numpy array, and t is an array of times.
	Returns a Numpy array.
	"""
	n = a.shape[0]
	nt = t.shape[0]
	las = len(a.shape)
	# print 'a=', a.shape, a
	if las == 1:
		# print 'a.shape=', a.shape
		a = numpy.transpose([a])
	m = a.shape[1]
	ii = numpy.around(t)
	assert ii.shape == t.shape

	# print 't=', t
	# print 'n=', n, 'ii=', ii
	if not numpy.alltrue((ii>=0) * (ii<=n-1)):
		idxmin = numpy.argmin(t)
		idxmax = numpy.argmax(t)
		raise IndexError, "Index out of range: min=%g[index=%d,int=%d] max=%g[index=%d,int=%d] vs. %d" % (t[idxmin], idxmin, ii[idxmin], t[idxmax], idxmax, ii[idxmax], n)

	nearestT = ii.astype(numpy.int)
	assert nearestT.shape == t.shape

	nearestA = numpy.take(a, nearestT, axis=0)
	assert nearestA.shape == (nt,m)
	deltaT = t - nearestT
	assert deltaT.shape == nearestT.shape
	isupport = numpy.where((deltaT>=0)*(nearestT<n-1)+(nearestT<=0), 1, -1) + nearestT
	assert isupport.shape == (nearestA.shape[0],)
	assert isupport.shape == nearestT.shape
	assert isupport.shape == deltaT.shape
	# print 'a=',a 
	# print 'isupport=', isupport
	support = numpy.take(a, isupport, axis=0)
	assert support.shape == (nt,m)
	if len(a.shape) > 1:
		assert nearestA.shape == (nt,m)
		deltaA = (deltaT/(isupport-nearestT))[:,numpy.newaxis] * (support-nearestA)
		assert deltaA.shape[1] == a.shape[1]
	else:
		deltaA = (deltaT/(isupport-nearestT)) * (support-nearestA)
	assert deltaA.shape == nearestA.shape
	rv = nearestA + deltaA
	if las == 1:
		return rv[:,0]
	return rv


def _test_interp1():
	a = numpy.array([[1.0], [2.0]])
	# a = numpy.array([1.0, 2.0], numpy.double)
	t = numpy.array([0.5])
	q = interp(a, t)
	assert q.shape == (1,1)
	assert numpy.sum(numpy.absolute(interp(a, t) - [1.5])) < 1e-3


def split_into_clumps(x, threshold, minsize=1):
	"""This reports when the signal is above the threshold.
	@param x: a signal
	@type x: L{numpy.ndarray}, one-dimensional.
	@param threshold: a threshold.
	@type threshold: float
	@returns: [(start, stop), ...] for each region ("clump") where C{x>threshold}.
	"""
	assert len(x.shape) == 1
	gt = numpy.greater(x, threshold)
	chg = gt[1:] - gt[:-1]
	nz = numpy.nonzero(chg)[0]
	rv = []
	if gt[0]:
		last = 0
	else:
		last = None
	for i in nz:
		if last is None:
			last = i+1
		else:
			if minsize > 0:
				rv.append( (last, i+1) )
			last = None
	if last is not None and gt.shape[0]>=last+minsize:
		rv.append( (last, gt.shape[0]) )
	return rv


def _test_split_into_clumps():
	tmp = split_into_clumps(numpy.array([0,0,0,1,0,0]), 0.5)
	assert tmp == [(3,4)]
	tmp = split_into_clumps(numpy.array([1,0,0]), 0.5)
	assert tmp == [(0,1)]
	tmp = split_into_clumps(numpy.array([0,0,0,1]), 0.5)
	assert tmp == [(3,4)]
	tmp = split_into_clumps(numpy.array([1]), 0.5)
	assert tmp == [(0,1)]
	tmp = split_into_clumps(numpy.array([0]), 0.5)
	assert tmp == []
	tmp = split_into_clumps(numpy.array([1]), 0.5, minsize=2)
	assert tmp == []
	tmp = split_into_clumps(numpy.array([0,0,0,1,1,0]), 0.5, minsize=2)
	assert tmp == [(3,5)]


BLOCK = 8192

def block_stdev(x):
	"""This is just a alternative implementation of
	the standard deviation of each channel, but it is designed
	in a block-wise fashion so the total memory usage is
	not large.
	"""
	m = (x.shape[0]+BLOCK-1)//BLOCK
	avg = numpy.zeros((m, x.shape[1],))
	sum2 = numpy.zeros((x.shape[1],))
	nn = numpy.zeros((m,))
	i = 0
	j = 0
	while i < x.shape[0]:
		n = min(BLOCK, x.shape[0]-i)
		tmp = numpy.sum(x[i:i+BLOCK], axis=0)/n
		avg[j,:] = tmp
		numpy.add(sum2, numpy.sum((x[i:i+BLOCK]-tmp)**2, axis=0), sum2)
		nn[j] = n
		i += BLOCK
		j += 1
	assert j == m
	avgavg = numpy.sum(nn[:,numpy.newaxis]*avg, axis=0)/float(x.shape[0])
	for i in range(m):
		numpy.add(sum2, nn[i]*(avg[i,:]-avgavg)**2, sum2)
	return numpy.sqrt(sum2/float(x.shape[0]-1))


# def _test_block_stdev():
	# n = BLOCK//2
	# while n < 4*BLOCK:
		# x = numpy.zeros((n,2))
		# x[0,0] = 1.0
		# x[0,1] = 2.0
		# x[1,0] = -1.0
		# avg2 = 2.0/float(n)
		# a2 = math.sqrt(((2-avg2)**2+(n-1)*avg2**2)/float(n-1))
		# answer = [math.sqrt(2/float(n-1)), a2]
		# calc = avg_dev(x)
		# assert numpy.sum(numpy.absolute(numpy.log(calc/answer))) < 0.001
		# n = (n*3)//2 + 17


def convolve(x, kernel):
	"""This is basically like numpy.convolve(x, kernel, 1) except that it
	properly handles the case where the kernel is longer than the data.
	"""
	assert len(kernel.shape) == 1
	assert len(x.shape) == 1, "Have not tried multidimensional data yet."
	if x.shape[0] > kernel.shape[0]:
		return numpy.convolve(x, kernel, 1)
	n = kernel.shape[0] - x.shape[0] + 1
	z = numpy.zeros((n,))
	xx = numpy.concatenate((z, x, z), axis=0)
	return numpy.convolve(xx, kernel, 1)[n:n+x.shape[0]]


if __name__ == '__main__':
	_test_split_into_clumps()
	_test_argmax()
	_test_interp1()
	_test_Poisson()
	_test_N_mean_ad()
	_test_N_median_across()
	# _test_block_stdev()
	_test_split_into_clumps()
